#!/bin/sh
exec ${PERL-perl} -Swx $0 ${1+"$@"}

#!/usr/bin/perl -w
#   userinfo.pl: admin user management
#	 Author: chifeng <chifeng At gmail.com>
#	   Date: 2007-11-28 15:39:00
#      Homepage: http://www.extmail.org
#	Version: 0.2
#
#
use vars qw($DIR);

BEGIN {
    my $path = $0;
    if ($path =~ s/tools\/[0-9a-zA-Z\-\_]+\.pl$//) {
	if ($path !~ /^\//) {
	    $DIR = "./$path";
	} else {
            $DIR = $path;
        }
    } else {
        $DIR = '../';
    }
    unshift @INC, $DIR .'libs';
};

# use strict; # developing env
no warnings; # production environment
use POSIX qw(strftime);
use Ext::Mgr;
use Ext::Utils qw(untaint);
use CmdTools;
use Ext::DateTime qw(time2epoch epoch2time);
use Getopt::Long;
use Switch;
use IO::File;
require $DIR.'/tools/setid.pl';

my $ctx = CmdTools->new( config => $DIR . '/webman.cf', directory => $DIR );
my $c = \%Ext::Cfg;
my $mgr = $ctx->ctx; # backend object
my $basedir = $c->{SYS_MAILDIR_BASE};
my %opt = ();
my $set_user=$c->{SYS_DEFAULT_UID};
my $set_group=$c->{SYS_DEFAULT_GID};

Getopt::Long::Configure('no_ignore_case');
Getopt::Long::GetOptions(\%opt,
	'mode=s',
	'file|f=s',
	'managername=s',
	'password|p=s',
	'changepwd|c=i',
	'name|n=s',
	'active=i',
	'expire|e=s',
	'question|q=s',
	'answer=s',
	'adddomains=s',
	'deldomains=s',
	'domains=s',
	'setgid=s',
	'setuid=s',
	'quiet|qq',
) or exit 1;

sub usage {
    print qq~
   Usage: $0 --mode=add,badd,del,bdel,list,show,mod
			--file=/path/to/filename.csv
			--managername=managername
			--password=******
			--changepwd=1|0
			--name=name
			--active=1|0
			--expire="2008-08-18"
			--question="what are you doing?"
			--answer="working"
			--adddomains="domain1.tld,domain2.tld,..."
			--deldomains="domain1.tld,domain2.tld,..."
			--domains="domain1.tld,domain2.tld,..."
			--setgid=vgroup
			--setuid=vuser
			--quiet or -qq [hide information bar, useful for batch job]
           $0 help
~;
    exit;
}
if($opt{setgid} && $opt{setuid}){
    $set_user = $opt{setuid};
    $set_group = $opt{setgid};
}
set_gid($set_user);
set_uid($set_group);

sub adduser {
    my $username = $_[0];
    my $uid = $_[1];
    my $password = $_[2];
    my $domains = $_[3];

    if($mgr->get_manager_info($username)){
	print "$username user exist!\n";
	exit;
    }else{
	my @dms = split(/,/,$domains);
	foreach my $d (@dms){
	    if(!($mgr->get_domain_info($d))){
	        print "$d domain no exist!\n";
		exit;
	    }
	}
	my $createdate = strftime("%Y-%m-%d %H:%M:%S", localtime);
	my $expiredate = '0000-00-00'; # default to unlimited/auto

        $domains =~ s#,# #g;
	my $rc = $mgr->add_manager(
	    manager => $username,
	    cn => $uid,
	    expire => $expiredate,
	    create => strftime("%Y-%m-%d %H:%M:%S", localtime),
	    active => $opt{active} ? $opt{active} : 0,
	    domain => $domains,
	    question => $opt{question} ? $opt{question} : '',
	    answer => $opt{answer} ? $opt{answer} : '',
	    disablepwdchange => $opt{changepwd} ? $opt{changepwd} : 0,
	    type => 'postmaster',
	    passwd => $password,
	);
	if($rc){
	    return 0;
	}else{
	    return 1;
	}
    }
}

sub add {
    if(!($opt{managername} && $opt{password} && $opt{domains})){
	print "Parameter Error!\n";
	exit;
    }else{
	my $username = $opt{managername};
	my $uid = $opt{managername};
	if($opt{name}){
	    $uid = $opt{name};
	}
	my $password = $opt{password};
	my $domains = $opt{domains};
	my $rv = adduser $username,$uid,$password,$domains ;
	if ($rv == 1){
	    print "$username\n";
	    print "----------------------------------\n";
	    print "Adminname: $username\n";
	    print " Password: $password\n";
	    print "     Type: postmaster\n";
	    print "  Domains: $domains\n";
	    print "   Status: Add Success!\n";
	}else{
	    print "$username add faild!\n";
	}
    }
}

sub badd {
    if($opt{file}){
	if( -e $opt{file} ){
	    my @info;
	    my @rv;
	    open(BAF, "< $opt{file}")
		or die "Can't open $opt{file} !\n";
	    while(<BAF>){
		chomp;
		@info = split(/ /, untaint($_));
		my $rv = adduser $info[0],$info[0],$info[1],$info[2];
		if($rv == 1){
		    print "$info[0] add success !\n";
		}else{
		    print "$info[0] add faild !\n";
		}
	    }
	    close BAF;
	}else{
	    print "$opt{file} file no exist!\n";
	}
    }else{
	print "\nPlease input a text file\n";
    }
}

sub deluser {
    my $username = $_[0];
    if($mgr->get_manager_info($username)){
	if(!($mgr->delete_manager($username))){
	    print "$username Delete Success!\n" 
	}else{
	    print "$username Delete faild!\n";
	}
    }else{
	print "$username no exist !\n";
    }
}

sub del {
    if($opt{managername}){
	deluser $opt{managername};
    }else{
	print "\nPlease input a admin name to delete!\n";
    }
}

sub bdel {
    if($opt{file}){
	if( -e $opt{file} ){
	    open(BDF, "< $opt{file}")
		or die "Can't open $opt{file} !\n";
	    while(<BDF>){
		chomp $_;
		deluser $_;
	    }
	    close BDF;
	}else{
	    print "$opt{file} file no exist!\n";
	}
    }else{
	print "\nPlease input a text file\n";
    }
}


sub list {
    my $all = $mgr->get_managers_list || [];
    print "User\tName\tUserType\tExpire\t\tActive\n" unless $opt{quiet};
    print "---------------------------------------------------------\n" unless $opt{quiet};
    for(my $i=0; $i<scalar @$all; $i++) {          
        my $e = $all->[$i];
        print "$e->{manager}\t$e->{cn}\t$e->{type}\t$e->{expire}\t$e->{active}\n";
    }
}

sub show {
    my $dms = [];
    if($opt{managername}){
	print "$opt{managername}\n";
	print "----------------------------------------\n";
	if(my $minfo = $mgr->get_manager_info($opt{managername})){
	    if($minfo->{type} eq 'admin'){
		push @$dms,'ALL Domains';
	    }else{
		$dms = $minfo->{domain};
	    }
	    print "            Type: $minfo->{type}\n";
	    print "        Username: $minfo->{manager}\n";
	    print "            Name: $minfo->{cn}\n";
	    print "          Active: $minfo->{active}\n";
	    print "DisablePWDchange: $minfo->{disablepwdchange}\n";
	    print "        Question: $minfo->{question}\n";
	    print "          Answer: $minfo->{answer}\n";
	    print "      CreateDate: $minfo->{create}\n";
	    print "      ExpireDate: $minfo->{expire}\n";
	    print "         Domains: ";
	    foreach my $rv (@$dms){
		print "$rv ";
	    }
	    print "\n";
	}else{
	    print "$opt{managername} no exist !\n";
	}
    }else{
	print "\nPlease input a admin name to show!\n";
    }
}

sub mod {
    if(!($opt{managername})){
	print "\nPlease input a manager name!\n";
	exit;
    }
    my $manager = $opt{managername};
    my $rv = $mgr->get_manager_info($manager);
    if(!$rv) {
	print "$manager no exist!\n";
	exit;
    }
    my $name = $opt{name} ? $opt{name} : $rv->{cn};
    my $changepwd = defined $opt{changepwd} ? $opt{changepwd} : $rv->{disablepwdchange};
    my $active = defined $opt{active} ? $opt{active} : $rv->{active};
    my @ttime = split(/ /,$rv->{expire});
    my $time = $ttime[1];
    my $expire = defined $opt{expire} ? $opt{expire}." ".$time : $rv->{expire};
    my $question = defined $opt{question} ? $opt{question} : $rv->{question};
    my $answer = defined $opt{answer} ? $opt{answer} : $rv->{answer};
    my $password = defined $opt{password} ? $opt{password} : undef;
    my $alldomains = $rv->{domain};
    if($opt{adddomains}){
	my @idomains = split(/,/,$opt{adddomains});
	foreach my $idm (@idomains){
	    if(!$mgr->get_domain_info($idm)){
		print "$idm domain no exist!\n";
		exit;
	    }
	}
	push @$alldomains,@idomains;
    }
    if($opt{deldomains}){
	my @rdomains = split(/,/,$opt{deldomains});
	my $i = 0;
	foreach my $domain (@rdomains){
	    for (;$i < scalar @$alldomains ; $i++){
		last if (@$alldomains[$i] eq $domain);
	    }
	    splice(@$alldomains,$i,1);
	    $i=0;
	}
    }
    if($opt{domains}){
	my @setdomains = split(/,/,$opt{domains});
	foreach my $setdm (@setdomains){
	    if(!$mgr->get_domain_info($setdm)){
		print "$setdm domain no exist!\n";
		exit;
	    }
	}
    @$alldomains = @setdomains ;
    }
    my %hash = ();
    my @alldomains2;
    foreach my $member (@$alldomains){
	$hash{$member} = 1;
    }
    foreach (keys %hash){
	push (@alldomains2,$_);
    }
    my $adomains = join(" ",@alldomains2);
	print "$manager\n";
	print "----------------------------------\n";
	print "      Manager: $manager\n";
	print "         Name: $name\n";
	print "     Password: $password\n" if($password);
	print "       Expire: $expire\n";
	print "    Changepwd: $changepwd\n";
	print "       Active: $active\n";
	print "     Question: $question\n";
	print "       Answer: $answer\n";
	print "      Domains: $adomains\n";
	print "       Status: Modify success!\n";

    $mgr->modify_manager(
	manager => $manager,
	cn => $name,
	expire => $expire,
	active => $active,
	domain => $adomains,
	question => $question,
	answer => $answer,
	disablepwdchange => $changepwd,
	passwd => $password,
    );
}

if($opt{mode}){
    switch ($opt{mode}){
	case "add" { add(); }
	case "badd" { badd(); }
	case "del" { del(); }
	case "bdel" { bdel(); }
	case "list" { list(); }
	case "show" { show(); }
	case "mod" { mod(); }
	case "help" { usage(); }
	else { usage(); }
    }
}else{
    usage ();
}

