#!/bin/sh
exec ${PERL-perl} -Swx $0 ${1+"$@"}

#!/usr/bin/perl -w
#   userinfo.pl: alias management
#	 Author: chifeng <chifeng At gmail.com>
#	   Date: 2008-01-06 15:39:00
#      Homepage: http://www.extmail.org
#	Version: 0.2
#
#
use vars qw($DIR);

BEGIN {
    my $path = $0;
    if ($path =~ s/tools\/[0-9a-zA-Z\-\_]+\.pl$//) {
	if ($path !~ /^\//) {
	    $DIR = "./$path";
	} else {
            $DIR = $path;
        }
    } else {
        $DIR = '../';
    }
    unshift @INC, $DIR .'libs';
};

# use strict; # developing env
no warnings; # production environment
use POSIX qw(strftime);
use Ext::Mgr;
use CmdTools;
use Ext::DateTime qw(time2epoch epoch2time);
use Getopt::Long;
use Switch;
use IO::File;
require $DIR.'/tools/setid.pl';

my $ctx = CmdTools->new( config => $DIR . '/webman.cf', directory => $DIR );
my $c = \%Ext::Cfg;
my $mgr = $ctx->ctx; # backend object
my $basedir = $c->{SYS_MAILDIR_BASE};
my %opt = ();
my $set_user=$c->{SYS_DEFAULT_UID};
my $set_group=$c->{SYS_DEFAULT_GID};

Getopt::Long::Configure('no_ignore_case');
Getopt::Long::GetOptions(\%opt,
	'mode|m=s',
	'file=s',
	'aliasname=s',
	'domain=s',
	'goto=s',
	'active=i',
	'setgid=s',
	'setuid=s',
	'quiet|qq',
) or exit 1;

sub usage {
    print qq~
   Usage: $0 --mode=add,del,list,show,mod
			--file=/path/to/filename
			--aliasname=aliasname\@domain.tld
			--domain=domain.tld
			--goto=username\@domain.tld
			--active=1|0
			--setgid=vgroup
			--setuid=vuser
			--quiet or -qq [hide information bar, useful for batch job]
           $0 help
~;
    exit;
}
if($opt{setgid} && $opt{setuid}){
    $set_user = $opt{setuid};
    $set_group = $opt{setgid};
}
set_gid($set_user);
set_uid($set_group);

sub addalias {
    my $aliasname = $_[0];
    my ($alias,$domain) = split(/@/,$aliasname);
    my $goto = $_[1];
    my $createdate = strftime("%Y-%m-%d %H:%M:%S", localtime);

    my $rc = $mgr->add_alias(
	alias => "$alias\@$domain",
	domain => $domain,
	goto => $goto,
	active => $opt{active} ? $opt{active} : 0,
	create => $createdate,
    );
    if($rc){
	return 0;
    }else{
	return 1;
    }
}

sub add {
    if($opt{aliasname}){
	my $aliasname = $opt{aliasname};
	my $goto = $opt{goto};
	if(!($opt{goto})){
	    print "goto can not Empty!\n";
	    exit;
	}
	if(addalias $aliasname,$goto){
	    print "Aliasname\tGoto\n";
	    print "-----------------------------\n";
	    print " Alias: $aliasname\n";
	    print "  Goto: $goto\n";
	    print "Status: Success!\n";
	}else{
	    print "Alias $aliasname add faild!\n";
	}
    }else{
	print "\nPlease input a alias for add!\n";
    }
}

sub delalias {
    my $aliasname = $_[0];
    if($mgr->get_alias_info($aliasname)){
	if($mgr->delete_alias($aliasname)){
	    print "$aliasname Deleted Faild!\n";
	}else{
	    print "$aliasname Deleted Success!\n";
	}
    }else{
	print "$aliasname no exist !\n";
    }
}

sub del {
    if($opt{aliasname}){
	delalias $opt{aliasname};
    }else{
	print "\nPlease input a alias address to delete!\n";
    }
}

sub list {
    if($opt{domain}){
	my $domain = $opt{domain};
	if(!($mgr->get_domain_info($domain))){
	    print "$domain no exists!\n";
	    exit ;
	}
	my $ul = $mgr->get_aliases_list($domain);
	print "Aliasname\t\tGoto\t\tActive\n" unless $opt{quiet};
	print "-----------------------------------------------------------------------\n" unless $opt{quiet};
	foreach my $u (@$ul) {
	    my $r = $u->{goto};
	    my $goto = ref $r ? join(',', @$r) : $r;
	    print "$u->{alias}\t$goto\t$u->{active} \n";
	}
    }else{
	print "\nPlease input a domain name !\n";
    }
}

sub show {
    my $dms = [];
    if($opt{aliasname}){
	print "$opt{aliasname}\n";
	print "----------------------------------------\n";
	if(my $ul = $mgr->get_alias_info($opt{aliasname})){
	    my $r = $ul->{goto};
	    my $goto = ref $r ? join(',', @$r) : $r;
	    print "       Type: Alias\n";
	    print "      Email: $ul->{alias}\n";
	    print "       Goto: $goto\n";
	    print "     Domain: $ul->{domain}\n";
	    print "     Active: $ul->{active}\n";
	}else{
	    print "$opt{aliasname} no exist !\n";
	}
    }else{
	print "\nPlease input a aliasname to show!\n";
    }
}

sub mod {
    if(!($opt{aliasname})){
	print "\nPlease input a user name for modify!\n";
	exit;
    }
    my $alias = $opt{aliasname};
    my $ul = $mgr->get_alias_info($alias);
    if(!$ul){
	print "user $alias not exists!\n";
	exit ;
    }

    my $active = defined $opt{active} ? $opt{active} : $ul->{active};
    if($ul){
	my $ogoto = ref $ul->{goto} ? @{$ul->{goto}} : $ul->{goto};
	my $goto = $opt{goto} ? $opt{goto} : $ogoto;
	print "$alias\n";
	print "---------------------------------\n";
	print " Alias: $alias\n";
	print "  Goto: $goto\n";
	print "Active: $active\n";

	$mgr->modify_alias(
	    alias => $alias,
	    goto => $goto,
	    active => $active,
	);
	print "\nAlias $alias modify success!\n";
    }

}

if($opt{mode}){
    switch ($opt{mode}){
	case "add" { add(); }
	case "del" { del(); }
	case "list" { list(); }
	case "show" { show(); }
	case "mod" { mod(); }
	case "help" { usage(); }
	else { usage(); }
    }
}else{
    usage ();
}
