#!/bin/sh
exec ${PERL-perl} -Swx $0 ${1+"$@"}

#!/usr/bin/perl -w
# vim: set ci et ts=4 sw=4:
#   userinfo.pl: domain name management
#	     Author: chifeng <chifeng At gmail.com>
#	       Date: 2007-11-28 15:39:00
#   Last Update: 2008-02-02 22:31:08 by hzqbbc
#      Homepage: http://www.extmail.org
#	Version: 0.2
#
#
use vars qw($DIR);

BEGIN {
    my $path = $0;
    if ($path =~ s/tools\/[0-9a-zA-Z\-\_]+\.pl$//) {
	if ($path !~ /^\//) {
	    $DIR = "./$path";
	} else {
            $DIR = $path;
        }
    } else {
        $DIR = '../';
    }
    unshift @INC, $DIR .'libs';
};

# use strict; # developing env
no warnings; # production environment
use POSIX qw(strftime);
use Ext::Mgr;
use CmdTools;
use Ext::DateTime qw(time2epoch epoch2time);
#use Ext::MgrApp qw(gen_domain_hashdir);
use Getopt::Long;
use Switch;
use IO::File;
use Data::Dumper;
require $DIR.'/tools/setid.pl';

my %opt = (); # parameters
my $ctx = CmdTools->new( config => $DIR . '/webman.cf', directory => $DIR );
my $c = \%Ext::Cfg;
my $mgr = $ctx->ctx; # backend object
my $set_user=$c->{SYS_DEFAULT_UID};
my $set_group=$c->{SYS_DEFAULT_GID};

Getopt::Long::Configure('no_ignore_case');
Getopt::Long::GetOptions(\%opt,
    'mode|m=s',
    'domainname|d=s',
    'description=s',
    'maxuser=i',
    'maxalias=i',
    'maxquota=i',
    'maxndquota=i',
    'expire|e=s',
    'active|a=i',
    'signup|s=i',
    'transport=s',
    'accountquota=i',
    'accountndquota=i',
    'accountlifetime=s',
    'accountservices=s',
    'setgid=s',
    'setuid=s',
    'quiet|qq',
) or exit 1;

sub usage {
    print qq~
  Usage: $0 --mode=add,del,list,show,mod
			--domainname=domain.tld
			--description="A test domain"
			--maxuser=100
			--maxalias=100
			--maxquota=5
			--maxndquota=5
			--expire="2008-08-18"
			--active=1|0
			--signup=1|0
			--transport="mx1.extmail.org"
			--accountquota=100
			--accountndquota=100
			--accountlifetime=1y
			--accountservices=smtp,smtpd,pop3,imap,webmail,netdisk
			--setgid=vgroup
			--setuid=vuser
			--quiet or -qq [hide information bar, useful for batch job]
           $0 help
~;
    exit;
}
if($opt{setgid} && $opt{setuid}){
    $set_user = $opt{setuid};
    $set_group = $opt{setgid};
}
set_gid($set_user);
set_uid($set_group);

sub add {
    if( !$opt{domainname} ){
	    print "Please input a domain name to add!\n\n";
	    exit;
    }
    my $domainname = $opt{domainname};
    my $description = $opt{domainname};
    if( defined $opt{description} ){
	$description = $opt{description};
    }
    my $hashdirpath = "empty";
    my $maxuser = $c->{SYS_DEFAULT_MAXUSERS};
    if(defined $opt{maxuser}){
	$maxuser = $opt{maxuser};
    }
    my $maxalias = $c->{SYS_DEFAULT_MAXALIAS};
    if(defined $opt{maxalias}){
	$maxalias = $opt{maxalias};
    }
    #maxquota,maxndquota,default_quota,default_ndquota
    my $maxquota = defined $opt{maxquota} ? $opt{maxquota} : $c->{SYS_DEFAULT_MAXQUOTA};
    $maxquota = $maxquota * 1024 * 1024;
    $maxquota = $maxquota."S";

    my $maxndquota = defined $opt{maxndquota} ? $opt{maxndquota} : $c->{SYS_DEFAULT_MAXNDQUOTA};
    $maxndquota = $maxndquota * 1024 * 1024;
    $maxndquota = $maxndquota."S";
    my $transport = "empty";
    if(defined $opt{transport}){
	$transport = $opt{transport};
    }
    my $can_signup = 0;
    if(defined $opt{signup}){
	$can_signup = $opt{signup};
    }
    my $default_quota = defined $opt{accountquota} ? $opt{accountquota} : $c->{SYS_USER_DEFAULT_QUOTA};
    $default_quota = $default_quota * 1024 * 1024;
    $default_quota = $default_quota."S";

    my $default_ndquota = defined $opt{accountndquota} ? $opt{accountndquota} : $c->{SYS_USER_DEFAULT_NDQUOTA};
    $default_ndquota = $default_ndquota * 1024 * 1024;
    $default_ndquota = $default_ndquota."S";

    my $default_expire = defined $opt{accountlifetime} ? $opt{accountlifetime} : $c->{SYS_USER_DEFAULT_EXPIRE};

    my $createdate = strftime("%Y-%m-%d %H:%M:%S", localtime);
    my $expiredate = '0000-00-00'; # set to unlimited/auto
    my $service = defined $opt{accountservices} ? $opt{accountservices} : $c->{SYS_DEFAULT_SERVICES};

    my %services= (
	    pwdchange=>0,
	    smtpd=>1,
	    smtp=>1,
	    webmail=>1,
	    netdisk=>1,
	    imap=>1,
	    pop3=>1
	);
    my @sv = split(/,/,$service);
    foreach (@sv) {
	    if (exists $services{$_}){
	        $services{$_}=0;
	    }
    }
    my $active = defined $opt{active} ? $opt{active} : 0;

    if($mgr->get_domain_info($domainname)){
	print "$domainname domain exist!\n";
	exit;
    }else{
	print "  $domainname\n";
	print "--------------------------------\n";
	print "          Domain: $domainname\n";
	print "     Description: $description\n";
	print "      ExpireDate: $expiredate\n";
	print "        MaxAlias: $maxalias\n";
	print "        MaxUsers: $maxuser\n";
	print "        MaxQuota: $maxquota\n";
	print "      MaxNDQuota: $maxndquota\n";
	print "       Transport: $transport\n";
	print "      Can Signup: $can_signup\n";
	print "    DefaultQuota: $default_quota\n";
	print "  DefaultNDQuota: $default_ndquota\n";
	print "   DefaultExpire: $default_expire\n";
	print "    DisableSmtpd: $services{smtpd}\n";
	print "     DisableSmtp: $services{smtp}\n";
	print "  DisableWebmail: $services{webmail}\n";
	print "  DisableNetdisk: $services{netdisk}\n";
	print "     DisableIMAP: $services{imap}\n";
	print "     DisablePOP3: $services{pop3}\n";
	print "          Active: $active\n";
	print "          Status: Add Successful!\n";
	print "\n";
	my $rc = $mgr->add_domain(
	    domain => $domainname,
            description => $description,
            hashdirpath => $hashdirpath,
            maxusers => $maxuser,
	    maxalias => $maxalias,
	    maxquota => $maxquota,
	    maxndquota => $maxndquota,
	    transport => $transport,
    	    can_sign => $can_signup,
	    default_quota => $default_quota,
	    default_ndquota => $default_ndquota,
	    default_expire => $default_expire,
	    disablesmtpd => $services{smtpd},
	    disablesmtp => $services{smtp},
	    disablewebmail => $services{webmail},
	    disablenetdisk => $services{netdisk},
	    disablepop3 => $services{pop3},
	    disableimap => $services{imap},
	    expire => $expiredate,
	    create => $createdate,
	    active => $active,
	    );
    }
}

sub del {
    if($opt{domainname}){
        if($mgr->get_domain_info($opt{domainname})){
	        if(!($mgr->delete_domain($opt{domainname}))){
	            print "$opt{domainname} Delete Success!\n" 
	        }else{
	            print "$opt{domainname} Delete faild!\n";
	        }
        }else{
	        print "$opt{domainname} no exist !\n";
        }
    }else{
	    print "Please input a domainname to delete!\n";
	    exit;
    }
}

sub list {
    my $all = $mgr->get_domains_list || [];
    print "User\tCreate\tExpire\t\tSignup\tActive\n" unless $opt{quiet};
    print "---------------------------------------------------------\n" unless $opt{quiet};
    for(my $i=0; $i<scalar @$all; $i++) {          
        my $e = $all->[$i];
        print "$e->{domain}\t$e->{create}\t$e->{expire}\t$e->{can_signup}\t$e->{active}\n";
    }
}

sub show {
    if(!$opt{domainname}){
        print "Please input a domain name\n";
        exit 1;
    }

	print "----------------------------------------\n";
	if(my $minfo = $mgr->get_domain_info($opt{domainname})){
	    print "          Domain: $minfo->{domain}\n";
	    print "     Description: $minfo->{description}\n";
	    print "      CreateDate: $minfo->{create}\n";
	    print "      ExpireDate: $minfo->{expire}\n";
	    print "     HashDirPath: $minfo->{hashdirpath}\n";
	    print "        MaxAlias: $minfo->{maxalias}\n";
	    print "        MaxUsers: $minfo->{maxusers}\n";
	    print "        MaxQuota: $minfo->{maxquota}\n";
	    print "      MaxNDQuota: $minfo->{maxndquota}\n";
	    print "       Transport: $minfo->{transport}\n";
	    print "      Can Signup: $minfo->{can_signup}\n";
	    print "    DefaultQuota: $minfo->{default_quota}\n";
	    print "  DefaultNDQuota: $minfo->{default_ndquota}\n";
	    print "   DefaultExpire: $minfo->{default_expire}\n";
	    print "    DisableSmtpd: $minfo->{disablesmtpd}\n";
	    print "     DisableSmtp: $minfo->{disablesmtp}\n";
	    print "  DisableWebmail: $minfo->{disablewebmail}\n";
	    print "  DisableNetdisk: $minfo->{disablenetdisk}\n";
	    print "     DisableIMAP: $minfo->{disableimap}\n";
	    print "     DisablePOP3: $minfo->{disablepop3}\n";
	    print "          Active: $minfo->{active}\n";
	    print "\n";
	}else{
	    print "$opt{domainname} no exist !\n";
	}
}

sub mod {
    if(!$opt{domainname}){
	print "\nPlease input a domain name!\n";
	exit;
    }
    my $domainname = $opt{domainname};
    my $ul = $mgr->get_domain_info($domainname);
    if(!($ul)){
	print "Domain $domainname not exist!\n";
	exit;
    }
    my $description = defined $opt{description} ? $opt{description} : $ul->{description};
    my $maxuser = defined $opt{maxuser} ? $opt{maxuser} : $ul->{maxusers};
    my $maxalias = defined $opt{maxalias} ? $opt{maxalias} : $ul->{maxalias};
    my $maxquota = $ul->{maxquota};
    if(defined $opt{maxquota}){
        $maxquota = $opt{maxquota}*1024*1024;
        $maxquota = $maxquota."S";
    }
    my $maxndquota = $ul->{maxndquota};
    if(defined $opt{maxndquota}){
        $maxndquota = $opt{maxndquota}*1024*1024;
        $maxndquota = $maxndquota."S";
    }
    my $transport = $opt{transport} ? $opt{transport} : $ul->{transport};
    my $can_signup = defined $opt{signup} ? $opt{signup} : $ul->{can_signup};
    my $default_quota = defined $opt{accountquota} ? $opt{accountquota} : $ul->{default_quota};
    my $default_ndquota = defined $opt{accountndquota} ? $opt{accountndquota} : $ul->{default_ndquota};
    my $default_expire = defined $opt{accountlifetime} ? $opt{accountlifetime} : $ul->{default_expire};
    my $expiredate = $opt{expire} ? $opt{expire} : $ul->{expire};
    my %services = (
	'pwdchange' => 1,
	'smtpd' => 1,
	'smtp' => 1,
	'webmail' => 1,
	'netdisk' => 1,
	'pop3' => 1,
	'imap' => 1,
    );
    #enable services,
    if ($opt{accountservices}) {
	my @sve = split(/,/,$opt{accountservices});
	foreach(@sve){
	    if (exists $services{$_}){
		$services{$_} = 0;
	    }
	}
    }
    my $active = defined $opt{active} ? $opt{active} : $ul->{active};
    print "  $domainname\n";
    print "--------------------------------\n";
    print "          Domain: $domainname\n";
    print "     Description: $description\n";
    print "      ExpireDate: $expiredate\n";
    print "        MaxAlias: $maxalias\n";
    print "        MaxUsers: $maxuser\n";
    print "        MaxQuota: $maxquota\n";
    print "      MaxNDQuota: $maxndquota\n";
    print "       Transport: $transport\n";
    print "      Can Signup: $can_signup\n";
    print "    DefaultQuota: $default_quota\n";
    print "  DefaultNDQuota: $default_ndquota\n";
    print "   DefaultExpire: $default_expire\n";
    print "    DisableSmtpd: $services{smtpd}\n";
    print "     DisableSmtp: $services{smtp}\n";
    print "  DisableWebmail: $services{webmail}\n";
    print "  DisableNetdisk: $services{netdisk}\n";
    print "     DisableIMAP: $services{imap}\n";
    print "     DisablePOP3: $services{pop3}\n";
    print "          Active: $active\n";
    print "\n";
    my $rc = $mgr->modify_domain(
	domain => $domainname,
	description => $description,
	maxusers => $maxuser,
	maxalias => $maxalias,
	maxquota => $maxquota,
	maxndquota => $maxndquota,
	transport => $transport,
	can_signup => $can_signup,
	default_quota => $default_quota,
	default_ndquota => $default_ndquota,
	default_expire => $default_expire,
	disablesmtpd => $services{smtpd},
	disablesmtp => $services{smtp},
	disablewebmail => $services{webmail},
	disablenetdisk => $services{netdisk},
	disablepop3 => $services{pop3},
	disableimap => $services{imap},
	expire => $expiredate,
	active => $active,
    );
}

if($opt{mode}){
    switch ($opt{mode}){
	case "add" { add(); }
	case "del" { del(); }
	case "list" { list(); }
	case "show" { show(); }
	case "mod" { mod(); }
	case "help" { usage(); }
	else { usage(); }
    }
}else{
    usage ();
}

