#!/bin/sh
exec ${PERL-perl} -Swx $0 ${1+"$@"}

#!/usr/bin/perl -w
# vim: set ci et ts=4 sw=4:
#   userinfo.pl: user managerment
#	     Author: chifeng <chifeng At gmail.com>
#	       Date: 2008-01-02 17:49:00
#   Last update: 2008-02-01 08:15:00 by hzqbbc
#      Homepage: http://www.extmail.org
#	    Version: 0.3
#
#
use vars qw($DIR);

BEGIN {
    my $path = $0;
    if ($path =~ s/tools\/[0-9a-zA-Z\-\_]+\.pl$//) {
	if ($path !~ /^\//) {
	    $DIR = "./$path";
	} else {
            $DIR = $path;
        }
    } else {
        $DIR = '../';
    }
    unshift @INC, $DIR .'libs';
};

# use strict; # developing env
no warnings; # production environment
use POSIX qw(strftime);
use Ext::Mgr;
use Ext::Utils qw(untaint);
use CmdTools;
use Ext::DateTime qw(time2epoch epoch2time);
use Getopt::Long;
use Switch;
use IO::File;
require $DIR.'/tools/setid.pl';

my %opt = (); # parameters
my $ctx = CmdTools->new( config => $DIR . '/webman.cf', directory => $DIR );
my $c = \%Ext::Cfg;
my $mgr = $ctx->ctx; # backend object
my $basedir = $c->{SYS_MAILDIR_BASE};
my $dir = $c->{SYS_CONFIG};
my $set_user=$c->{SYS_DEFAULT_UID};
my $set_group=$c->{SYS_DEFAULT_GID};

Getopt::Long::Configure('no_ignore_case');
Getopt::Long::GetOptions(\%opt,
    'mode|m=s',
    'file|f=s',
    'username|u=s',
    'domain|d=s',
    'password|p=s',
    'name=s',
    'question=s',
    'answer=s',
    'gid=i',
    'uid=i',
    'expire|e=s',
    'quota|q=s',
    'ndquota=s',
    'active|a=i',
    'services=s',
    'setgid=s',
    'setuid=s',
    'quiet|qq',
) or exit 1;

sub usage {
    print qq~
    Usage: $0 --mode=add,badd,del,bdel,list,show,mod,help
			--file=/path/to/filename.csv  #if mode is:badd,bdel
			--username=username\@domain.tld
			--domain=domain.tld
			--password=******
			--name=name
			--question="What are you doing?"
			--answer="Working"
			--gid=gid
			--uid=uid
			--expire="2009-09-17"
			--quota=100   # Unit: MB
			--ndquota=100   # Unit: MB
			--active=1|0
			--services=smtp,smtpd,pop3,imap,webmail,netdisk,pwdchange
			--setgid=vgroup
			--setuid=vuser
			--quiet or -qq [hide infomation bar, useful for batch job]
~;
    exit;
}
if($opt{setgid} && $opt{setuid}){
    $set_user = $opt{setuid};
    $set_group = $opt{setgid};
}
set_gid($set_user);
set_uid($set_group);

sub adduser {
    my $email = $_[0];
    my $password = $_[1];
    my $quota = $_[2]; # MB
    my $netdiskquota = $_[3]; # MB
    my ($user,$domain) = split(/@/,$email);
    my $uid = $c->{SYS_DEFAULT_UID};
    my $gid = $c->{SYS_DEFAULT_GID};

    my $createdate = strftime("%Y-%m-%d %H:%M:%S", localtime);
    my $expiredate = '0000-00-00'; # default to unlimited/auto
    my $question = defined $opt{question} ? $opt{question} : "";
    my $answer = defined $opt{answer} ? $opt{answer} : "";

    my $path = $domain."/".$user;
    $netdiskquota ||= $c->{SYS_USER_DEFAULT_NDQUOTA};
    $netdiskquota = num2quota($netdiskquota);

    $quota ||= $c->{SYS_USER_DEFAULT_QUOTA};
    $quota = num2quota($quota);

    my $service = $c->{SYS_DEFAULT_SERVICES};
    my %disable= (
	    pwdchange=>0,
	    smtpd=>1,
	    smtp=>1,
	    webmail=>1,
	    netdisk=>1,
	    imap=>1,
	    pop3=>1
    );
    my @sv = split(/,/,$service);
    foreach (@sv) {
	if (exists $disable{$_}){
	    $disable{$_}=0;
        }
    }

    my $ul = $mgr->get_user_info($email);
    my $ula = $mgr->get_alias_info($email);
    if($ul || $ula){
	    print "\n$email exists!\n";
	    exit;
    }else{
        my $rv = $mgr->add_user(
	    mail => "$user\@$domain",
	    domain => $domain,
	    uid => $user,
	    cn => $user,
	    uidnumber => $uid,
	    gidnumber => $gid,
	    create => strftime("%Y-%m-%d %H:%M:%S", localtime),
	    expire => $expiredate,
	    passwd => $password,
	    quota => $quota,
	    question => $question,
	    answer => $answer,
	    mailhost => "",
	    maildir => "$path/Maildir/",
	    homedir => $path,
	    netdiskquota => $netdiskquota,
	    active => 1,
	    disablepwdchange => $disable{pwdchange},
	    disablesmtpd => $disable{smtpd},
	    disablesmtp => $disable{smtp},
	    disablewebmail => $disable{webmail},
	    disablenetdisk => $disable{netdisk},
	    disablepop3 => $disable{pop3},
	    disableimap => $disable{imap}
        );
	    system("$dir/tools/maildirmake.pl $basedir/$path/Maildir/");
	    if($rv){
	        return 0;
	    }else{
	        return 1;
	    }
    }
}

sub add {
    if($opt{username}){
	my $email = $opt{username};
	my $password = $opt{password};
	my $quota = $c->{SYS_USER_DEFAULT_QUOTA}; #default size got from webman.cf

    if ($opt{quota}) {
        $quota = $opt{quota};
    }

	if(!($password)) {
	    print "Password can not Empty!\n";
	    exit 1;
	}

	if(adduser $email,$password,$quota){
	    print "$email\n";
	    print "-------------------------------\n" unless $opt{quiet};
	    print "    User: $email \n";
	    print "Password: $password \n";
	    print "  Status: Success!\n";
	}else{
	    print "$email add faild!\n";
	}
        }else{
	        print "\nPlease input username and some infomation for add !\n";
        }
}

sub badd {
    if(!$opt{file}){
        print "Please input a text file\n";
        exit 1;
    }
	if( -e $opt{file} ){
	    my @info;
	    my $rv;
	    open(BAF,"< $opt{file}")
		or die "Can't open $opt{file} !\n";
	    while(<BAF>){
		chomp;
		@info = split(/ /, untaint($_));
        $rv = adduser $info[0],$info[1],$info[2];
		if($rv eq 1){
		    print "$info[0] add success !\n";
		}else{
		    print "$info[0] add faild !\n";
		}
	    }
	    close BAF;
	}else{
	    print "$opt{file} file no exist!\n";
	}
}

sub deluser {
    my $username = $_[0];
    if($mgr->get_user_info($username)){
	    if($mgr->delete_user($username)){
	        print "$username Deleted Faild!\n";
	    }else{
	        print "$username Deleted Success!\n";
	    }
    }else{
	    print "$username no exist !\n";
    }
}

sub del {
    if($opt{username}){
	    deluser $opt{username};
    }else{
	    print "Please input a email address to delete!\n";
    }
}

sub bdel {
    if(!$opt{file}){
        print "Please input a text file\n";
        exit 1;
    }
	if( -e $ARGV[1] ){
	    open(BDF, "< $opt{file}")
		or die "Can't open $opt{file} !\n";
	    while(<BDF>){
		chomp $_;
		deluser untaint($_);
	    }
	    close BDF;
	}else{
	    print "$opt{file} file no exist!\n";
	}
}

sub list {
    if(!$opt{domain}){
        print "Please input a domain name\n";
        exit 1;
    }
	my $domain = $opt{domain};
	if(!($mgr->get_domain_info($domain))){
	    print "$domain no exists!\n";
	    exit ;
	}
	my $ul = $mgr->get_users_list($domain);
	print "Email\t\tCommonName\tExpire\tActive\n" unless $opt{quiet};
	print "-------------------------------------------------------------\n" unless $opt{quiet};
	foreach my $u (@$ul) {
	    print "$u->{mail}\t$u->{cn}\t$u->{expire}\t$u->{active} \n";
	}
}

sub show {
    my $dms = [];
    if(!$opt{username}) {
        print "Please input a username to show\n";
        exit 1;
    }


	print "----------------------------------------\n";
	if( my $ul = $mgr->get_user_info($opt{username})){
	    print "            Type: User\n";
	    print "           Email: $ul->{mail}\n";
	    print "      CommonName: $ul->{cn}\n";
	    print "        Password: $ul->{passwd}\n";
	    print "       UidNumber: $ul->{uidnumber}\n";
	    print "       GidNumber: $ul->{gidnumber}\n";
	    print "           Quota: $ul->{quota}\n";
	    print "        Question: $ul->{question}\n";
	    print "          Answer: $ul->{answer}\n";
	    print "    NetdiskQuota: $ul->{netdiskquota}\n";
	    print "          Active: $ul->{active}\n";
	    print "         Maildir: $basedir\/$ul->{maildir}\n";
	    print "         Homedir: $basedir\/$ul->{homedir}\n";
	    print "         Routing: $ul->{mailhost}\n";
	    print "      CreateDate: $ul->{create}\n";
	    print "      ExpireDate: $ul->{expire}\n";
	    print "DisablePWDChange: $ul->{disablepwdchange}\n";
	    print "    DisableSMTPD: $ul->{disablesmtpd}\n";
	    print "     DisableSMTP: $ul->{disablesmtp}\n";
	    print "  DisableWebmail: $ul->{disablewebmail}\n";
	    print "  DisableNetDisk: $ul->{disablenetdisk}\n";
	    print "     DisablePOP3: $ul->{disablepop3}\n";
	    print "     DisableIMAP: $ul->{disableimap}\n";
my $oldpwd = $opt{password};
my $type = $c->{SYS_CRYPT_TYPE};
my $p2 = $ul->{passwd};
my $pwd = Ext::Passwd->new( fallback_scheme => $type );
my $rv = $pwd->verify($oldpwd, $p2);
print "$rv \n";
	}else{
	    print "$opt{username} no exist !\n";
	}
}

sub num2quota {
    # sys_quota_type, valid type: vda|courier
    #SYS_QUOTA_TYPE = courier
    my $quota = $_[0]; # must be number
    $quota = $quota * 1024 * 1024;
    my $type = $c->{SYS_QUOTA_TYPE} || 'vda';
    if ($type eq 'vda') {
	return $quota ? $quota : '0';
    } else {
	return $quota."S";
    }
}

sub quota2num {
    my $self = shift;
    my $quota = $_[0];
    $quota =~ s/S$//i;
    return $quota;
}

sub mod {
    if(!$opt{username}){
	    print "\nPlease input a user name for modify!\n";
	    exit 1;
    }
    my $email = $opt{username};
    my ($user,$domain) = split(/\@/,$email);
    my $password = $opt{password};
    my $quota = $opt{quota};
    my $ndquota = $opt{ndquota};
    my $expiredate = $opt{expire};
    my $ul = $mgr->get_user_info($email);

    if(!$ul){
	    print "user $email not exists!\n";
	    exit ;
    }

    my $active = (defined $opt{active} ? $opt{active} : $ul->{active});
    my $question = defined $opt{question} ? $opt{question} : $ul->{question};
    my $answer = defined $opt{answer} ? $opt{answer} : $ul->{answer};

    #smtp,smtpd,pop3,imap,webmail,netdisk,pwdchange
    #PRIORITY: 1:disable, 2:enable
    my %services = (
     	'pwdchange' => $ul->{disablepwdchange},
		'smtpd' => $ul->{disablesmtpd},
		'smtp' => $ul->{disablesmtp},
		'webmail' => $ul->{disablewebmail},
		'netdisk' => $ul->{disablenetdisk},
		'pop3' => $ul->{disablepop3},
		'imap' => $ul->{disableimap}
	);
    #enable services,
    if ($opt{services}) {
	#smtp,smtpd,pop3,imap,webmail,netdisk,pwdchange
	#PRIORITY: 1:disable, 2:enable
	%services = (
       	'pwdchange' => 1,
		'smtpd' => 1,
		'smtp' => 1,
		'webmail' => 1,
		'netdisk' => 1,
		'pop3' => 1,
		'imap' => 1
	);
        my @sve = split(/,/,$opt{services});
        foreach(@sve){
	    if (exists $services{$_}){
	        $services{$_} = 0;
	    }
        }
    }

    if($ul){
	my $uid = $ul->{uidnumber};
	my $gid = $ul->{gidnumber};
	my @ttime = split(/ /,$ul->{expire});
	my $time = $ttime[1];
	$expiredate = $expiredate ? $expiredate." ".$time : $ul->{expire};
	$quota = $quota ? num2quota $quota : $ul->{quota};
	$password = $password ? $password : undef;
	my $netdiskquota = $ndquota ? num2quota $ndquota : $ul->{netdiskquota};
	print "$user\@$domain"."\n";
	print "-------------------------------\n";
	print "            Type: User\n";
	print "           Email: $user\n";
	print "          Domain: $domain\n";
	print "             UID: $uid\n";
	print "             GID: $gid\n";
	print "      ExpireDate: $expiredate\n";
	print "        Password: $password\n" if($password);
	print "           Quota: $quota\n";
	print "        Question: $question\n";
	print "          Answer: $answer\n";
	print "    NetDiskQuota: $netdiskquota\n";
	print "          Active: $active\n";
	print "Disablepwdchange: $services{pwdchange}\n";
	print "    DisableSmtpd: $services{smtpd}\n";
	print "     DisableSmtp: $services{smtp}\n";
	print "  DisableWebmail: $services{webmail}\n";
	print "     DisablePop3: $services{pop3}\n";
	print "     DisableImap: $services{imap}\n";
	print "  DisableNetDisk: $services{netdisk}\n";

	my $rc = $mgr->modify_user(
	    user => "$user\@$domain",
	    domain => $domain,
	    cn => $user,
	    uidnumber => $uid,
	    gidnumber => $gid,
	    expire => $expiredate,
	    passwd => $password,
	    quota => $quota,
	    question => $question,
	    answer => $answer,
	    netdiskquota => $netdiskquota,
	    active => $active,
	    disablepwdchange => $services{pwdchange},
	    disablesmtpd => $services{smtpd},
	    disablesmtp => $services{smtp},
	    disablewebmail => $services{webmail},
	    disablenetdisk => $services{netdisk},
	    disablepop3 => $services{pop3},
	    disableimap => $services{imap},
	);
	print "\nUSER $email modify success!\n";
    }
}

if($opt{mode}){
    switch ($opt{mode}){
	case "add" { add(); }
	case "badd" { badd(); }
	case "del" { del(); }
	case "bdel" { bdel(); }
	case "list" { list(); }
	case "show" { show(); }
	case "mod" { mod(); }
	case "help" { usage(); }
	else { usage(); }
    }
}else{
    usage ();
}
